==> org-lsst-ccs-subsystem-console-5.0.10-sources.jar: org/lsst/ccs/subsystems/console/py4j/Py4JServer.java <==
package org.lsst.ccs.subsystems.console.py4j;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;
import org.lsst.ccs.BusMaster;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.command.annotations.Argument;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.commons.annotations.Persist;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.messaging.NetworkUtilities;
import org.lsst.ccs.scripting.CCS;
import org.lsst.ccs.scripting.ScriptingStatusBusListener;
import org.lsst.ccs.scripting.ScriptingStatusBusMessage;
import org.lsst.ccs.scripting.ScriptingStatusBusMessageFilter;
import org.lsst.ccs.scripting.ScriptingSubsystemWrapper;
import org.lsst.ccs.utilities.conv.InputConversionEngine;
import org.lsst.ccs.utilities.conv.TypeUtils;

import py4j.GatewayServer;

public class Py4JServer extends BusMaster implements HasLifecycle {
    
    @Persist
    private final List<Py4jGatewayServer> gateways = new ArrayList<>();
    private final static String LOCALHOST_NANE = "default-localhost";
        
    public Py4JServer() {
        super("py4j-server", AgentInfo.AgentType.SERVICE);
    }

    @Override
    public void shutdown() {
        for (Py4jGatewayServer gw : gateways) {
            gw.stopServer();
        }
    }

    @Override
    public void start() {
        getAgentPersistenceService().load();
        try {
            if (gateways.isEmpty()) {
                createGateway();
            } else {
                for ( Py4jGatewayServer server : gateways ) {
                    server.startServer();
                }
            }
        } catch (UnknownHostException e) {
            throw new RuntimeException(e);
        }
    }
        
    @Command
    public void createGateway() throws UnknownHostException {
        createGateway(LOCALHOST_NANE, 25333, LOCALHOST_NANE, 25334);
    }
    
    @Command(description = "starts a gateway")
    public void createGateway(
            @Argument(description = "the name of this gateway") String name,
            @Argument(description = "the listening port")int port,
            @Argument(description = "the python client's callback address") String callbackAddress,
            @Argument(description = "the python client's callback port") int callbackPort) throws UnknownHostException {
        Py4jGatewayServer gateway = new Py4jGatewayServer(name, port, callbackAddress, callbackPort);
        gateway.startServer();
        gateways.add(gateway);
        getAgentPersistenceService().persistNow();
    }

    @Command(description = "restart a gateway")
    public void restartGateway(@Argument(description = "the name of the gateway") String name)  {
        for ( Py4jGatewayServer server : gateways ) {
            if ( server.getName().equals(name) ) {
                server.stopServer();
                server.startServer();
                return;
            }
        }
        throw new IllegalArgumentException("No Gateway is available with name "+name);
    }

    @Command(description = "remove a gateway")
    public void removeGateway(@Argument(description = "the name of the gateway") String name)  {        
        Iterator<Py4jGatewayServer> iter = gateways.iterator();
        while ( iter.hasNext() ) {
            Py4jGatewayServer server = iter.next();
            if ( server.getName().equals(name) ) {
                iter.remove();
                getAgentPersistenceService().persistNow();
                return;
            }
        }
        throw new IllegalArgumentException("No Gateway is available with name "+name);
    }

    
    @Command
    public String listGateways() {
        StringBuilder sb = new StringBuilder();
        if (gateways.isEmpty()) {
            sb.append("No gateways are currently available");
        } else {
            for (Py4jGatewayServer gw : gateways) {
                sb.append(gw.toPrettyString()).append("\n");
            }
        }
        return sb.toString();
    }
    
    public static class Py4jGatewayServer {
        private GatewayServer server;
        private InetAddress address;
        private InetAddress callbackAddress;
        private final Map<String,String> parameters = new HashMap<>();
        
        private final static String NAME = "name", SERVER_PORT="serverPort", CALLBACK_ADDRESS="callbackAddress", CALLBACK_PORT="callbackPort";
                      
        
        public Py4jGatewayServer(String stringRepresentation) {
            Map<String,String> pars = (Map<String,String>)InputConversionEngine.convertArgToType(stringRepresentation, Map.class);
            initializeServer(pars.get(NAME), Integer.valueOf(pars.get(SERVER_PORT)), pars.get(CALLBACK_ADDRESS), Integer.valueOf(pars.get(CALLBACK_PORT)));
        }
        
        Py4jGatewayServer() throws UnknownHostException {
            this(LOCALHOST_NANE, 25333, LOCALHOST_NANE, 25334);
        }
        
        Py4jGatewayServer(String name, int port, String callbackAddress, int callbackPort) {
            initializeServer(name, port, callbackAddress, callbackPort);
        }
        
        private final void initializeServer(String name, int port, String callbackAddress, int callbackPort) {
            address = getGatewayAddress(name);
            this.callbackAddress = getCallbackAddress(callbackAddress);
            server = new GatewayServer(this, port,callbackPort, address, this.callbackAddress,0,0,null);                
            parameters.put(NAME, name);
            parameters.put(SERVER_PORT, String.valueOf(port));
            parameters.put(CALLBACK_ADDRESS, callbackAddress);
            parameters.put(CALLBACK_PORT, String.valueOf(callbackPort));            
        }
        
        private InetAddress getGatewayAddress(String name) {
            try {
                if (name.equals(LOCALHOST_NANE)) {
                    return InetAddress.getLoopbackAddress();
                } else {
                    return NetworkInterface.getByName(NetworkUtilities.getMainInterfaceName()).getInterfaceAddresses().get(0).getAddress();
                }
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
        
        private InetAddress getCallbackAddress(String name) {
            try {
                if (name.equals(LOCALHOST_NANE)) {
                    return InetAddress.getLoopbackAddress();
                } else {
                    return InetAddress.getByName(name);
                }
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
        
        public String getName() {
            return parameters.get(NAME);
        }
        
        public int getPort() {
            return Integer.valueOf(parameters.get(SERVER_PORT));
        }
        
        private void startServer() {
            server.start();
            System.out.println("Started Gateway Server on "+server.getAddress()+":"+server.getPort()+" and callback "+server.getCallbackClient().getAddress()+" "+server.getCallbackClient().getPort());
        }
        
        private void stopServer() {
            server.shutdown();
        }
        
        @Override
        public String toString() {
            return TypeUtils.stringify(parameters);
        }

        public String toPrettyString() {
            return getName() + " Java(" + server.getAddress() + ":" + server.getPort() + ") -> Python(" + server.getCallbackClient().getAddress()+" "+server.getCallbackClient().getPort() + ")";
        }
        
        public void initializeScriptingEnvironment() throws UnknownHostException {
            server.resetCallbackClient(callbackAddress, Integer.valueOf(parameters.get(CALLBACK_PORT)));
            CCS.initializeScriptingEnvironment();
        }
        
        public void shutdownScriptingEnvironment() throws Exception {
            CCS.shutdownScriptingEnvironment();
        }
        
        public ScriptingSubsystemWrapper attachSubsystem(String target) {
            return CCS.attachSubsystem(target);
        }
        
        public void setThrowExceptions(boolean throwExceptions) {
            CCS.setThrowExceptions(throwExceptions);
        }
        
        public boolean getThrowExceptions() {
            return CCS.getThrowExceptions();
        }
        
        public void addStatusBusListener(ScriptingStatusBusListener listener) {
            CCS.addStatusBusListener(listener);
        }
        
        public void addStatusBusListener(ScriptingStatusBusListener listener, ScriptingStatusBusMessageFilter filter) {
            CCS.addStatusBusListener(listener, filter);
        }
        
        public void removeStatusBusListener(ScriptingStatusBusListener listener) {
            CCS.removeStatusBusListener(listener);
        }
        
        public Future<ScriptingStatusBusMessage> startListeningForStatusBusMessage(ScriptingStatusBusMessageFilter filter, int timeout) {
            return CCS.startListeningForStatusBusMessage(filter, timeout);
        }
        
    }
    
}

==> org-lsst-ccs-subsystem-console-6.2.9-20250701.214333-6-sources.jar: org/lsst/ccs/subsystems/console/py4j/Py4JServer.java <==
package org.lsst.ccs.subsystems.console.py4j;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.UnknownHostException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

import org.lsst.ccs.Subsystem;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.command.annotations.Argument;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.commons.annotations.Persist;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.messaging.NetworkUtilities;
import org.lsst.ccs.scripting.CCS;
import org.lsst.ccs.scripting.ScriptingStatusBusListener;
import org.lsst.ccs.scripting.ScriptingStatusBusMessage;
import org.lsst.ccs.scripting.ScriptingStatusBusMessageFilter;
import org.lsst.ccs.scripting.ScriptingSubsystemWrapper;
import org.lsst.ccs.utilities.conv.InputConversionEngine;
import org.lsst.ccs.utilities.conv.TypeUtils;

import py4j.GatewayServer;

public class Py4JServer extends Subsystem implements HasLifecycle {
    
    @Persist
    private final List<Py4jGatewayServer> gateways = new ArrayList<>();
    private final static String LOCALHOST_NANE = "default-localhost";
        
    public Py4JServer() {
        super("py4j-server", AgentInfo.AgentType.SERVICE);
    }

    @Override
    public void shutdown() {
        for (Py4jGatewayServer gw : gateways) {
            gw.stopServer();
        }
    }

    @Override
    public void start() {
        getAgentPersistenceService().load();
        try {
            if (gateways.isEmpty()) {
                createGateway();
            } else {
                for ( Py4jGatewayServer server : gateways ) {
                    server.startServer();
                }
            }
        } catch (UnknownHostException e) {
            throw new RuntimeException(e);
        }
    }
        
    @Command
    public void createGateway() throws UnknownHostException {
        createGateway(LOCALHOST_NANE, 25333, LOCALHOST_NANE, 25334);
    }
    
    @Command(description = "starts a gateway")
    public void createGateway(
            @Argument(description = "the name of this gateway") String name,
            @Argument(description = "the listening port")int port,
            @Argument(description = "the python client's callback address") String callbackAddress,
            @Argument(description = "the python client's callback port") int callbackPort) throws UnknownHostException {
        Py4jGatewayServer gateway = new Py4jGatewayServer(name, port, callbackAddress, callbackPort);
        gateway.startServer();
        gateways.add(gateway);
        getAgentPersistenceService().persistNow();
    }

    @Command(description = "restart a gateway")
    public void restartGateway(@Argument(description = "the name of the gateway") String name)  {
        for ( Py4jGatewayServer server : gateways ) {
            if ( server.getName().equals(name) ) {
                server.stopServer();
                server.startServer();
                return;
            }
        }
        throw new IllegalArgumentException("No Gateway is available with name "+name);
    }

    @Command(description = "remove a gateway")
    public void removeGateway(@Argument(description = "the name of the gateway") String name)  {        
        Iterator<Py4jGatewayServer> iter = gateways.iterator();
        while ( iter.hasNext() ) {
            Py4jGatewayServer server = iter.next();
            if ( server.getName().equals(name) ) {
                iter.remove();
                getAgentPersistenceService().persistNow();
                return;
            }
        }
        throw new IllegalArgumentException("No Gateway is available with name "+name);
    }

    
    @Command
    public String listGateways() {
        StringBuilder sb = new StringBuilder();
        if (gateways.isEmpty()) {
            sb.append("No gateways are currently available");
        } else {
            for (Py4jGatewayServer gw : gateways) {
                sb.append(gw.toPrettyString()).append("\n");
            }
        }
        return sb.toString();
    }
    
    public static class Py4jGatewayServer {
        private GatewayServer server;
        private InetAddress address;
        private InetAddress callbackAddress;
        private final Map<String,String> parameters = new HashMap<>();
        
        private final static String NAME = "name", SERVER_PORT="serverPort", CALLBACK_ADDRESS="callbackAddress", CALLBACK_PORT="callbackPort";
                      
        
        public Py4jGatewayServer(String stringRepresentation) {
            Map<String,String> pars = (Map<String,String>)InputConversionEngine.convertArgToType(stringRepresentation, Map.class);
            initializeServer(pars.get(NAME), Integer.valueOf(pars.get(SERVER_PORT)), pars.get(CALLBACK_ADDRESS), Integer.valueOf(pars.get(CALLBACK_PORT)));
        }
        
        Py4jGatewayServer() throws UnknownHostException {
            this(LOCALHOST_NANE, 25333, LOCALHOST_NANE, 25334);
        }
        
        Py4jGatewayServer(String name, int port, String callbackAddress, int callbackPort) {
            initializeServer(name, port, callbackAddress, callbackPort);
        }
        
        private final void initializeServer(String name, int port, String callbackAddress, int callbackPort) {
            address = getGatewayAddress(name);
            this.callbackAddress = getCallbackAddress(callbackAddress);
            server = new GatewayServer(this, port,callbackPort, address, this.callbackAddress,0,0,null);                
            parameters.put(NAME, name);
            parameters.put(SERVER_PORT, String.valueOf(port));
            parameters.put(CALLBACK_ADDRESS, callbackAddress);
            parameters.put(CALLBACK_PORT, String.valueOf(callbackPort));            
        }
        
        private InetAddress getGatewayAddress(String name) {
            try {
                if (name.equals(LOCALHOST_NANE)) {
                    return InetAddress.getLoopbackAddress();
                } else {
                    return NetworkInterface.getByName(NetworkUtilities.getMainInterfaceName()).getInterfaceAddresses().get(0).getAddress();
                }
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
        
        private InetAddress getCallbackAddress(String name) {
            try {
                if (name.equals(LOCALHOST_NANE)) {
                    return InetAddress.getLoopbackAddress();
                } else {
                    return InetAddress.getByName(name);
                }
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
        
        public String getName() {
            return parameters.get(NAME);
        }
        
        public int getPort() {
            return Integer.valueOf(parameters.get(SERVER_PORT));
        }
        
        private void startServer() {
            server.start();
            System.out.println("Started Gateway Server on "+server.getAddress()+":"+server.getPort()+" and callback "+server.getCallbackClient().getAddress()+" "+server.getCallbackClient().getPort());
        }
        
        private void stopServer() {
            server.shutdown();
        }
        
        @Override
        public String toString() {
            return TypeUtils.stringify(parameters);
        }

        public String toPrettyString() {
            return getName() + " Java(" + server.getAddress() + ":" + server.getPort() + ") -> Python(" + server.getCallbackClient().getAddress()+" "+server.getCallbackClient().getPort() + ")";
        }
        
        public void initializeScriptingEnvironment() throws UnknownHostException {
            server.resetCallbackClient(callbackAddress, Integer.valueOf(parameters.get(CALLBACK_PORT)));
            CCS.initializeScriptingEnvironment();
        }
        
        public void shutdownScriptingEnvironment() throws Exception {
            CCS.shutdownScriptingEnvironment();
        }
        
        public ScriptingSubsystemWrapper attachSubsystem(String target) {
            return CCS.attachSubsystem(target);
        }
        
        public void setThrowExceptions(boolean throwExceptions) {
            CCS.setThrowExceptions(throwExceptions);
        }
        
        public boolean getThrowExceptions() {
            return CCS.getThrowExceptions();
        }
        
        public void addStatusBusListener(ScriptingStatusBusListener listener) {
            CCS.addStatusBusListener(listener);
        }
        
        public void addStatusBusListener(ScriptingStatusBusListener listener, ScriptingStatusBusMessageFilter filter) {
            CCS.addStatusBusListener(listener, filter);
        }
        
        public void removeStatusBusListener(ScriptingStatusBusListener listener) {
            CCS.removeStatusBusListener(listener);
        }
        
        public Future<ScriptingStatusBusMessage> startListeningForStatusBusMessage(ScriptingStatusBusMessageFilter filter, Duration timeout) {
            return CCS.startListeningForStatusBusMessage(filter, timeout);
        }
        
    }
    
}
